﻿using System;
using Foundation;
using WatchConnectivity;

namespace WCSessionTest.iOS
{
    public class OnPhoneWatchSessionDelegate : WCSessionDelegate
    {
        #region Sincleton/Constructor
        public static OnPhoneWatchSessionDelegate Instance { get; private set; } = new OnPhoneWatchSessionDelegate();

        private OnPhoneWatchSessionDelegate()
        {
            Console.WriteLine("In OnPhoneWatchSessionDelegate constructor");
        }
        #endregion

        #region Public Methods
        public void SendMessage(string key, string value)
        {
            Console.WriteLine("OnPhoneWatchSessionDelegate->SendMessage(...) called");
            Console.WriteLine("key = {0}, value = {1}", key, value);

            NSString[] keys = { new NSString(key) };
            NSObject[] values = { new NSString(value) };
            var dict = new NSDictionary<NSString, NSObject>(keys, values);

            var replyHandler = new WCSessionReplyHandler((NSDictionary<NSString, NSObject> replyMessage) =>
            {
                Console.WriteLine("In OnPhoneWatchSessionDelegate->SendMessage(...)-> replyHandler");
                Console.WriteLine("replyMessage: {0}", replyMessage);
            });

            var errorHandler = new Action<NSError>((NSError error) =>
            {
                Console.WriteLine("In OnPhoneWatchSessionDelegate->SendMessage(...)-> errorHandler");
                Console.Write("ERROR sending message to watch: {0}", error);
            });

            WCSession.DefaultSession.SendMessage(dict, replyHandler, errorHandler);
        }
        #endregion

        #region WCSessionDelegate overrides
        public override void ActivationDidComplete(WCSession session, WCSessionActivationState activationState, NSError error)
        {
            Console.WriteLine("OnPhoneWatchSessionDelegate->ActivationDidComplete(...) called");
        }

        public override void SessionReachabilityDidChange(WCSession session)
        {
            Console.WriteLine("OnPhoneWatchSessionDelegate->SessionReachabilityDidChange(...) called");
        }

        public override void DidReceiveMessage(WCSession session, NSDictionary<NSString, NSObject> message)
        {
            Console.WriteLine("OnPhoneWatchSessionDelegate->DidReceiveMessage(...) called");

            if (message != null && message.Keys.Length > 0)
            {
                var key = message.Keys[0];
                var value = message[key];
                Console.WriteLine("Message received on phone: key = {0}, value = {1}", key, value);
            }
            else
            {
                Console.WriteLine("ERROR: Bad message received");
            }
        }
        #endregion
    }
}
